;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;This script used to calculate and plot the zonal mean variables from CAM;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;The terms zonal and meridional are used to describe directions on a globe;;;;;;;
;;;;;;;Zonal means "along a latitudinal circle" or "in the west–east direction";;;;;;;;
;;;;;;;while meridional means "along a longitudinal circle" (a.k.a. meridian) ;;;;;;;;;
;;;;;;;or "in the north–south direction" [2];;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  l_plot_cntl       = True  ; if True, plot the CNTL origianl field on the leftmost panel
  plot_contour      = False   ; if True, plot the contour in the figure
  plot_leftstr      = True    ; if True, the lefadd igure captions as left string
  plot_sig_test     = True

  datadir     = "../fig_data/"

  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  Refers      = (/"CLIM30","CLIM30ML"/)
  Groups      = (/"SSP245","SSP245ML",\
                  "SSP585","SSP585ML"/)
  LabList     = (/"SSP245","ML(SSP245)", \
                  "SSP585","ML(SSP585)"/)
  LabList(0::2) = LabList(0::2) + " ~F19~:~N~~F~ " + "CLIM"
  LabList(1::2) = LabList(1::2) + " ~F19~:~N~~F~ " + "ML(CLIM)"
  ngrps         = dimsizes(Groups)

  yst         = 1979
  yed         = 2014

  if((yed-yst+1).lt.5) then 
    plot_sig_test  = False 
  end if 

  PltNums     = (/"(a)","(b)", "(c)", "(d)"/)
  VarList     = (/"T850"/)
  VarName     = "T850"
  VarUnit     = "K"
  VarStr      = "Responses (~F33~D~N~~F~) in 850-hPa temperature (K)"
  VarSt1      = "Responses (~F33~D~N~~F~) in 850-hPa temperature (K)"
  nvars       = dimsizes(VarList)
  rstr        = "Annual mean" 
  plotname    = "fig_2d_map_delta_"+VarName

  do igp = 0, ngrps-1,1

   model = Groups(igp)
   if(isStrSubset(model,"ML")) then 
     obs   = Refers(1)
   else
     obs   = Refers(0)
   end if 
   fl0   = datadir+obs+"_"+VarList+"_"+yst+"-"+yed+".nc"
   f0    = addfile(fl0, "r")
   fl1   = datadir+model+"_"+VarList+"_"+yst+"-"+yed+".nc"
   f1    = addfile(fl1, "r")
 
   uboto = f0->$VarName$
   ubotm = f1->$VarName$

   guess     = 1                ; use zonal means
   is_cyclic = True             ; cyclic [global]
   nscan     = 1500             ; usually much less than this
   eps       = 1.e-2            ; variable dependent
   relc      = 0.6              ; relaxation coefficient
   opt       = 0                ; not used

   uboto     = where(ismissing(ubotm),uboto@_FillValue,uboto)
   ubotm     = where(ismissing(uboto),ubotm@_FillValue,ubotm)
   ubotd     = ubotm
   ubotd     = ubotm - uboto

  ;print(num(ismissing(uboto)))
  ;print(num(ismissing(ubotm)))
  
   xAve  = f0->$VarName$
   yAve  = f1->$VarName$
   xVar  = f0->variance
   yVar  = f1->variance
   sigr  = 0.05                                   ; critical sig lvl for r
   xN    = xAve@samples 
   yN    = yAve@samples 
   iflag = False                        ; population variance similar
   prob  = xAve
   prob  = (/ttest(xAve,xVar,xN, yAve,yVar,yN, iflag, False)/)
   prob  = where(prob.lt.sigr,prob,prob@_FillValue )
   delete([/xAve,yAve,xVar,yVar,sigr,xN,yN,iflag/])
 
   ;ubotm = where(lmsk.eq.0.0,ubotm@_FillValue,ubotm)
   ;uboto = where(lmsk.eq.0.0,uboto@_FillValue,uboto)
   ;prob  = where(lmsk.eq.0.0,prob@_FillValue,prob)
   ;ubotd = where(lmsk.eq.0.0,ubotd@_FillValue,ubotd)
 
   ;calculate the global/regional mean values
   latrg = f1->lat
   rad   = 4.*atan(1.)/180.    ;
   wgty  = cos(latrg*rad)      ; cosine weights
   gbmn  = wgt_areaave_Wrap(ubotd,   wgty, 1.0, 0)
   grms  = wgt_arearmse(uboto,ubotm, wgty, 1.0, 0)

   rstr1  = "Annual mean" ;"U (850hPa, m/s)";
   rstr2  = "Bias/RMSD: "+ sprintf("%.2f",gbmn) + "/" + sprintf("%.2f",grms)
   rstr2  = "RMSD: " + sprintf("%.2f",grms) + " " + VarUnit
   delete([/latrg,rad,wgty,gbmn,grms/])

   guess     = 1                ; use zonal means
   is_cyclic = True             ; cyclic [global]
   nscan     = 1500             ; usually much less than this
   eps       = 1.e-2            ; variable dependent
   relc      = 0.6              ; relaxation coefficient
   opt       = 0                ; not used
   poisson_grid_fill(uboto, is_cyclic, guess, nscan, eps, relc, opt)

   ;----------------------------------------------------------------------
   ; Plotting section
   ;----------------------------------------------------------------------
   if(igp.eq.0)then

    ;;;;;;;open a wks for plot;;;;;;;;;;;;;;;;;;;
    wtype = "pdf"
   ;wtype@wkPaperSize  = "A4"
   ;wtype@wkOrientation   = "landscape" ;;pictures displayed along the length
    wks1 = gsn_open_wks(wtype,plotname)
    print(" ")
    print("........................... ")
    print("plot : " + plotname+".pdf")
    print("........................... ")
    plot     = new(ngrps,"graphic")

    cmap0       = read_colormap_file("percent_11lev")
    cmap        = new((/1,4/),typeof(cmap0))
    cmap(0,:)   = 0 
    printVarSummary(cmap)
    cnlevels    = (/0.4,0.6,0.8,1.0,1.2,1.4,1.6,1.8,2.0,2.4,2.8/)
    colors      = (/cmap(0,:),cmap0(1,:),cmap0(2,:),cmap0(3,:),cmap0(4,:), \
                    cmap0(5,:),cmap0(6,:),cmap0(7,:),cmap0(8,:),cmap0(9,:),cmap0(10,:),cmap0(11,:)/)
   end if

   ;;;set up the plot information;;;;
   ;load "./var_share_colorbar.ncl"

;---Resources to share between both plots
   res                = True     ; Plot modes desired.
   res@gsnDraw        = False
   res@gsnFrame       = False
   res@gsnMaximize    = False     ; Maximize plot
   res@tmXTOn         = False
   res@tmYROn         = False

   BorderThick = 1.0
   res@tmBorderThicknessF    = BorderThick
   res@tmXBMajorThicknessF   = BorderThick
   res@tmXBMinorThicknessF   = BorderThick*0.5
   res@tmYLMajorThicknessF   = BorderThick
   res@tmYLMinorThicknessF   = BorderThick*0.5
   res@tmXTMajorThicknessF   = BorderThick
   res@tmXTMinorThicknessF   = BorderThick*0.5
   res@tmYRMajorThicknessF   = BorderThick
   res@tmYRMinorThicknessF   = BorderThick*0.5

   FontHeightF = 0.024
   res@tiMainFontHeightF    = FontHeightF
   res@tmYLLabelFontHeightF = FontHeightF
   res@tmXBLabelFontHeightF = FontHeightF

   mpres     = res
   res@gsnStringFontHeightF  = FontHeightF*1.5
   res@lbLabelFontHeightF    = FontHeightF*0.6
   res@lbTitleFontHeightF    = FontHeightF*0.8

   res@cnFillOn              = True     ; color plot desired
  ;res@cnFillPalette         = "ViBlGrWhYeOrRe";"testcmap" ;"NCV_jaisnd";"temp_19lev"
   res@cnLinesOn             = False    ; turn off contour lines
   res@cnLineLabelsOn        = False    ; turn off contour labels
  ;res@cnFillMode            = "RasterFill"   ; turn raster on
   res@cnMissingValFillColor = "White"

   res@lbLabelBarOn             = False   ; Will turn on in panel later
   res@pmLabelBarParallelPosF   = 0.50 ;61
   res@pmLabelBarOrthogonalPosF = 0.20
  ;res@lbOrientation            = "Horizonal" ; "vertical"  ; vertical label bar
   res@pmLabelBarHeightF        = 0.12 ;0.60
   res@pmLabelBarWidthF         = 0.65 ;0.06
   res@lbTitleString            = "~F33~D~N~~F~U (850hPa, m/s)"; 
  ;res@lbTitlePosition          = "Right"         ; title location
  ;res@lbTitleDirection         = "Across"        ; letter angle
  ;res@lbTitleAngleF            = 90.          ; title angle

   res@gsnCenterString    = ""
   res@gsnLeftString      = ""
   res@gsnRightString     = "" 
   res@tiMainString       = "" 

;---Control appearance of map.
    mpres@gsnMajorLatSpacing     = 30
    mpres@gsnMajorLonSpacing     = 45
    mpres@gsnMinorLatSpacing     = 15
    mpres@gsnMinorLonSpacing     = 15
   ;mpres@mpProjection           = "Robinson"       ; choose projection
    mpres@mpLabelsOn             = False
    mpres@mpPerimOn              = False
    mpres@mpFillOn               = False
    mpres@mpGeophysicalLineColor = "Grey23"  ; color of cont. outlines
    mpres@mpGeophysicalLineThicknessF = 1.0          ; thickness of outlines
   ;mpres@mpOutlineOn            = True
   ;mpres@mpOutlineDrawOrder     = "PostDraw"
   ;mpres@mpFillDrawOrder        = "Predraw"
   ;mpres@mpGridAndLimbOn        = True
   ;mpres@mpGridLatSpacingF      = 90.0
   ;mpres@mpGridLonSpacingF      = 180.0
   ;mpres@mpGridLineThicknessF   = 3.0
   ;mpres@mpGridAndLimbDrawOrder="PreDraw"
   ;mpres@mpMinLonF              = 0  ;-180.
   ;mpres@mpMaxLonF              = 360;180.
    mpres@mpCenterLonF           = 180.0
;   mpres@mpCenterLatF           = -90.     ; This is necessary to get the correct map
;   mpres@mpOceanFillColor       = "lightskyblue1"
;   mpres@mpLandFillColor        = "gray33"

   mpres@tmYLLabelsOn     = True
   mpres@tmYLOn           = True
   mpres@tmYROn           = False

   plot(igp) = gsn_csm_map(wks1,mpres)

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   res2=res
   res2@gsnAddCyclic                 = True
   res2@cnLevelSelectionMode         = "ExplicitLevels"  
   res2@cnFillColors                := colors
   res2@cnLevels                    := cnlevels
   res2@cnLinesOn                    = True    ; turn off contour lines
   res2@cnLineThicknessF             = 0.1
   res2@cnLineLabelsOn               = False   ; turn off contour labels
   res2@cnInfoLabelOn                = False
   res2@gsnLeftString                = PltNums(igp)+"  " + LabList(igp)
   res2@gsnLeftStringOrthogonalPosF  = 0.04
  ;res2@gsnLeftStringParallelPosF    = 0.2 
   res2@gsnRightString               = rstr2 
   res2@gsnRightStringOrthogonalPosF = 0.055
   plot_rgd1 = gsn_csm_contour(wks1,ubotd,res2)  
   overlay(plot(igp),plot_rgd1)

   if(plot_contour)
     res21 =    res2
     res21@gsnLeftString      = ""
     res21@gsnRightString     = ""
     res21@gsnCenterString    = ""
     res21@cnFillOn           = False
     res21@cnLinesOn          = True
     res21@cnLineLabelsOn     = False   ; turn off contour labels
     res21@cnInfoLabelOn      = False
     res21@cnLineLabelDensityF      = 2.0
     res21@cnLineLabelFontHeightF   = 0.018
     res21@cnLineLabelBackgroundColor     = -1
     res21@gsnContourLineThicknessesScale = 1.0
     res21@cnLineThicknessF      = 1.0
     res21@cnLineColor     = "Black"
     res21@gsnContourZeroLineThicknessF   = 2.
     res21@gsnContourNegLineDashPattern   = 1
     plot_cont1      = gsn_csm_contour(wks1,ubotd,res21)
     overlay(plot(igp),plot_cont1)
     delete([/res21/])
   end if

   if(plot_sig_test) then
      res21 = res2
      res21@gsnLeftString                = ""
      res21@gsnRightString               = ""
      res21@gsnCenterString              = ""
      res21@cnMissingValFillColor        = "Transparent"
      res21@cnFillOn                     = True
      res21@cnLinesOn                    = False
      res21@cnLineLabelsOn               = False       ; turn off contour labels
      res21@cnInfoLabelOn                = False
      res21@cnMonoFillColor              = True        ; default color is fground [black]
      res21@cnMonoFillPattern            = True
      res21@cnMonoFillScale              = True
      res21@cnFillPattern                = 17
      res21@cnFillScaleF                 = 0.8
      res21@cnFillDotSizeF               = 0.0005
      res21@lbLabelBarOn                 = False            
      plot_cont2 = gsn_csm_contour(wks1,prob,res21)
      overlay(plot(igp),plot_cont2)
      delete([/res21/])
   end if
   delete([/ubotm,ubotd/])
   delete([/res,res2,mpres/])
 end do ; end of experiment group loop

;---Specify y positions of each row of plots.
  nrows       = 1
  plot_height = 1./tofloat(nrows)
  ypos1  = 0.96     ; top row
  ypos2  = 0.64     ; middle row
  xpos   = (/0.05,0.315,0.57/)     ; top row

  dist_from_title_to_plot = 0.013
  ytitl1 = ypos1 + dist_from_title_to_plot
  ytitl2 = ypos2 + dist_from_title_to_plot

  pres                          = True
; pres@gsnPanelDebug            = True      ; prints debug info
  pres@gsnFrame                 = False
  pres@gsnMaximize              = False
  pres@gsnPanelMainFontHeightF  = 0.015
  pres@gsnPanelFigureStrings    = ""
  pres@gsnPanelFigureStringsFontHeightF  = 0.01

;---Top row
  pres1 = pres
  pres1@gsnPanelLabelBar         = False
  pres1@pmLabelBarParallelPosF   = -0.1
  pres1@pmLabelBarOrthogonalPosF = -0.02
  pres1@pmLabelBarHeightF        = 0.03 ;0.60
  pres1@pmLabelBarWidthF         = 0.22 ;0.06
  pres1@lbLabelFontHeightF       = FontHeightF*0.25
  pres1@lbTitleFontHeightF       = FontHeightF*0.30
  pres1@lbTitleString            = VarStr
  pres1@gsnPanelTop              = 1.0
  pres1@gsnPanelBottom           = 0.5
  pres1@gsnPanelYF              := (/ypos1,ypos1/)
  pres1@gsnPanelMainString       = ""
  pres1@gsnPanelMainPosYF        = ytitl1
  gsn_panel(wks1,plot(0:1),(/1,2/),pres1)

;---Bottom row
  pres2 = pres
  pres2@gsnPanelLabelBar         = True
  pres2@pmLabelBarParallelPosF   = 0.02
  pres2@pmLabelBarOrthogonalPosF = -0.02
  pres2@pmLabelBarHeightF        = 0.06 ;0.60
  pres2@pmLabelBarWidthF         = 0.90 ;0.06
  pres2@lbLabelFontHeightF       = FontHeightF*0.55
  pres2@lbTitleFontHeightF       = FontHeightF*0.60
  pres2@lbTitleString            = VarSt1
  pres2@gsnPanelTop              = 0.5
  pres2@gsnPanelBottom           = 0.0
  pres2@gsnPanelYF              := (/ypos2,ypos2/)
  pres2@gsnPanelMainString       = ""
  pres2@gsnPanelMainPosYF        = ytitl2    ; Y position of title
  gsn_panel(wks1,plot(2:3),(/1,2/),pres2)

  frame(wks1)


end
